.TH pkgmake 1
.SH NAME
pkgmake - make a Solaris SVR4 package
.SH SYNOPSIS
pkgmake [options] filesdir [destdir]
.SH DESCRIPTION
\fBpkgmake\fP is a simple utility for making Solaris SVR4 packages. It takes a
directory structure containing the files you want to include in the package and
produces a Solaris SVR4 package from it, resulting in a .pkg file suitable for
distribution and installation.

\fBfilesdir\fP must contain the files to install, in their final hierarchy,
with their final permissions. This is typically a temporary directory created
by \fB"make DESTDIR=... install"\fP.

In addition to the files, a \fBpkginfo\fP file must be present in
\fBfilesdir\fP. This file specifies the package's name, version and so on (see
example below).

There may also optionally be a \fBdepend\fP file listing other packages which
are required dependencies (see example), a \fBcompver\fP file listing versions
of the package which this version may replace while maintaining compatibility
(see example), and a \fBcopyright\fP file displayed during installation.

Finally, there may be scripts named \fBrequest\fP, \fBcheckinstall\fP,
\fBpreinstall\fP, \fBpostinstall\fP, \fBpreremove\fP and \fBpostremove\fP. The
meaning of the scripts should be self-evident from their names, except for
\fBrequest\fP which is used to ask the user for information during
installation, and thus is rarely used in practice (because any form of
interaction interferes with automated installations).

A \fBprototype\fP file is not required, but may be provided if you wish.
Normally the files in \fBfilesdir\fP are used to generate a \fBprototype\fP
file automatically. You should only need to provide one if you're doing
something very unusual.

Each of these files may have a filename extension if you wish (.sh for the
scripts, .txt for the rest).
.SH OPTIONS
.TP
.B -l | --long-name
Generate a package file using the full, long package name, eg:
ACMEcooltool-1.2.3-solaris-sparc.pkg. The default is to generate a shortened
filename which does not include the company prefix, eg:
cooltool-1.2.3-solaris-sparc.pkg. The actual package name remains in its full
form either way, this only affects the filename.
.TP
.B -o | --owners
Keep the ownership of the files and directories exactly as is. Normally,
\fBpkgmake\fP produces a package where files are installed owned by root and
system directories such as /usr and /var have their ownership left unchanged.
That's the desired behavior for the vast majority of packages. Use this option
if you need to have special ownership for your package's files once installed.
.TP
.B -v | --verbose
Be verbose, display the output of underlying commands.
.TP
.B -V | --version
Print the version number of \fBpkgmake\fP.
.SH EXAMPLES
A typical use from a shell is...
.nf

    $ make DESTDIR=installtemp install
    $ cp pkginfo installtemp
    $ pkgmake installtemp
    $ rm -rf installtemp

.fi
Another typical use is in a \fBMakefile\fP, allowing package creation to be
part of the build process via \fB"make package"\fP...
.nf

    INSTALLTEMP = installtemp

    package: all
        - rm -rf $(INSTALLTEMP)
        $(MAKE) DESTDIR=$(INSTALLTEMP) install
        cp pkginfo $(INSTALLTEMP)
        pkgmake $(INSTALLTEMP)
        rm -rf $(INSTALLTEMP)

.fi
A typical \fBpkginfo\fP file looks like...
.nf

    PKG="ACMEcooltool"
    NAME="Cool Tool"
    DESC="Program to make you really cool."
    VERSION="1.2.3"
    VENDOR="ACME Tools"
    EMAIL="cooltool@acme.com"
    ARCH="native"

.fi
Most of the fields in \fBpkginfo\fP are self-explanatory. Values for ARCH are
the outputs of \fBuname -p\fP (eg: sparc, i386), or the word \fBnative\fP which
will be replaced by the output of \fBuname -p\fP, or the word \fBall\fP for
hardware-independent packages, such as scripts and documentation.

A typical \fBdepend\fP file looks like...
.nf

    P   SUNWpng     Portable Network Graphics library
    P   SUNWzlib    Zip compression library

.fi
Be careful if you add version information to your \fBdepend\fP file (see
\fBdepend(4)\fP), because not all packages have accurate backward-compatibility
information (\fBcompver\fP files). You might end up with problems in the future
because you depend on an older version of a library when a newer, perfectly
compatible version is installed but doesn't list itself as being backwardly
compatible with the older version number. This is especially true for 3rd-party
packages. It's also not necessary to list any essential parts of the operating
system, such as SUNWcsr and SUNWcsu, since they must be installed to be running
Solaris in the first place.

A typical \fBcompver\fP file looks like...
.nf

    1.0
    1.1
    1.1.1
    1.2
    1.2.1
    1.2.2
    1.2.3

.fi
Normally a \fBcompver\fP file will be generated automatically using the
industry-standard 1.2.3 style of version numbering, including 1 and 1.2 style
shortenings. You don't need to provide one unless either your package's version
numbering scheme is unusual (eg: uses numbers over 99.99.99, uses dates/letters
other than the standard 1.2b3 or 1.2.3b4 style for alpha/beta versions), or
your package does \fBnot\fP offer backward compatibility with all previous
versions.
.SH SCRIPTS & BASEDIR
Scripts such as \fBpostinstall\fP should use the \fBBASEDIR\fP env var to refer
to files and directories, because the package may be being installed into a
different root directory structure from the one the \fBpkgadd\fP command is
being run in. In other words, \fBBASEDIR\fP is the root of the target
file system, which may not necessarily be /. In fact, during JumpStart or Live
Upgrade operations it definitely won't be.
.SH UPGRADES
Package upgrades are handled in a very inelegant fashion on Solaris. Rather
than simply upgrading a package to a newer version in-place as most people
would expect, on Solaris there are two approaches, both somewhat less than
ideal...
.TP
.B pkgadd then pkgrm
The natural approach for most people is to simply install the new version of
the package using \fBpkgadd(1M)\fP. Because this will write the new files over
the originals, it causes \fBpkgadd(1M)\fP to request confirmation, saying that
there are "conflicting" files, which is ugly but acceptable.

After installation of the new version, however, there will now actually be
\fBtwo\fP instances of the package installed on the system (the old \fBand\fP
the new), at least as far as the package database is concerned. I kid you not -
there really will be two instances of the package installed, even though in
terms of files on the system the new versions will have completely replaced the
old ones. The new package will be under the package name followed by a dot and
an instance number (2, 3 etc). This is quite unexpected for most people, who
would sensibly assume the new version had replaced the old one. Not so.

The old package should now actually be removed with \fBpkgrm(1M)\fP, which will
leave the new files in place because they belong to the new package.
.TP
.B pkgrm then pkgadd
The "better" approach is to first remove the existing package with
\fBpkgrm(1M)\fP, then install the new version with \fBpkgadd(1M)\fP.
Unfortunately, this will break any dependencies (temporarily), causing warnings
to be issued during the \fBpkgrm(1M)\fP, which is not a good installation
experience. Nevertheless, this is actually the recommended approach.
.PP
Exactly why Solaris hasn't added the "feature" of being able to upgrade to a
new version of a package in one step is a mystery which remains unsolved.
Solaris could learn a lot from Linux in this area.
.SH NOTES
\fBpkgmake\fP is not intended to cover every possibility, and does not expose
every last ounce of the package format's capabilities. It's intended to provide
a simple utility suitable for producing the 90% of packages which are just a
bunch of files and optionally some pre/post-install scripts. For complete
access to all of the SVR4 package format's capabilities, use the lower-level
\fBpkgmk(1)\fP and \fBprototype(4)\fP facilities directly.
.SH SEE ALSO
pkginfo(4), depend(4), compver(4), pkgadd(1M), pkgrm(1M), pkginfo(1M),
pkgchk(1M), pkgmk(1), prototype(4)
.SH WEB SITE
www.lighterra.com/pkgmake
.SH COPYRIGHT
Copyright (c) 2003-2018 Lighterra - Open Source (MPL & LGPL)
.SH AUTHOR
Jason Robert Carey Patterson <jason@lighterra.com>
